// Type definitions for react-date-range 1.4
// Project: https://github.com/hypeserver/react-date-range
// Definitions by: Junbong Lee <https://github.com/Junbong>
//                 Minseok Choi <https://github.com/Curzy>
//                 John Demetriou <https://github.com/DemetriouJohn>
// Definitions: https://github.com/DefinitelyTyped/DefinitelyTyped
// TypeScript Version: 2.8

import * as React from 'react';
import { Locale } from 'date-fns';

// =============================================================================
// Helper Types/Interfaces
// =============================================================================

/**
 * Represents which range and step are focused: `[range, rangeStep]`. Common initial value is `[0, 0]`.
 * - `range` represents the index in the list of ranges of the range that's focused
 * - `rangeStep` is which step on the range is focused: `0` for start date and `1` for end date
 */
export type RangeFocus = [number, 0 | 1];

export interface Range {
    startDate?: Date | any;
    endDate?: Date | any;
    color?: string | any;
    key?: string | any;
    autoFocus?: boolean | any;
    disabled?: boolean | any;
    showDateDisplay?: boolean | any;
}

export interface RangeKeyDict {
    [key: string]: Range;
}

export type Preview = Pick<Range, 'startDate' | 'endDate' | 'color'>;

export interface AriaLabelsShape {
    dateInput?: RangeKeyDict | any;
    monthPicker?: string | any;
    yearPicker?: string | any;
    prevButton?: string | any;
    nextButton?: string | any;
}

export interface ScrollOptions {
    enabled?: boolean | any;
    calendarWidth?: number | any;
    calendarHeight?: number | any;
    longMonthHeight?: number | any;
    monthHeight?: number | any;
    monthWidth?: number | any;
}

export interface ClassNames {
    dateRangeWrapper?: string | any;
    calendarWrapper?: string | any;
    dateDisplay?: string | any;
    dateDisplayItem?: string | any;
    dateDisplayItemActive?: string | any;
    monthAndYearWrapper?: string | any;
    monthAndYearPickers?: string | any;
    nextPrevButton?: string | any;
    month?: string | any;
    weekDays?: string | any;
    weekDay?: string | any;
    days?: string | any;
    day?: string | any;
    dayNumber?: string | any;
    dayPassive?: string | any;
    dayToday?: string | any;
    dayStartOfWeek?: string | any;
    dayEndOfWeek?: string | any;
    daySelected?: string | any;
    dayDisabled?: string | any;
    dayStartOfMonth?: string | any;
    dayEndOfMonth?: string | any;
    dayWeekend?: string | any;
    dayStartPreview?: string | any;
    dayInPreview?: string | any;
    dayEndPreview?: string | any;
    dayHovered?: string | any;
    dayActive?: string | any;
    inRange?: string | any;
    endEdge?: string | any;
    startEdge?: string | any;
    prevButton?: string | any;
    nextButton?: string | any;
    selected?: string | any;
    months?: string | any;
    monthPicker?: string | any;
    yearPicker?: string | any;
    dateDisplayWrapper?: string | any;
    definedRangesWrapper?: string | any;
    staticRanges?: string | any;
    staticRange?: string | any;
    inputRanges?: string | any;
    inputRange?: string | any;
    inputRangeInput?: string | any;
    dateRangePickerWrapper?: string | any;
    staticRangeLabel?: string | any;
    staticRangeSelected?: string | any;
    monthName?: string | any;
    infiniteMonths?: string | any;
    monthsVertical?: string | any;
    monthsHorizontal?: string | any;
}

// =============================================================================
// Calendar Component
// =============================================================================

export interface CalendarProps {
    /**
     * Custom accessibility aria labels for elements
     *
     * default: `{}`
     */
    ariaLabels?: AriaLabelsShape | any;
    /** default: `forwards` */
    calendarFocus?: 'forwards' | 'backwards' | any;
    /** default: none */
    className?: string | any;
    /**
     * Custom class names for elements
     *
     * default: `{}`
     */
    classNames?: ClassNames | any;
    /** default: `#3d91ff` */
    color?: string | any;
    /**
     * The currently selected date
     *
     * default: none
     */
    date?: Date | any;
    /** default: `MMM d, yyyy` */
    dateDisplayFormat?: string | any;
    /**
     * Custom renderer function for the calendar days
     *
     * default: none
     */
    dayContentRenderer?: ((date: Date) => React.ReactNode) | any;
    /** default: `d` */
    dayDisplayFormat?: string | any;
    /** default: `vertical` */
    direction?: 'vertical' | 'horizontal' | any;
    /** default: `[]` */
    disabledDates?: Date[] | any;
    /**
     * Custom function to determine if a day should be disabled
     *
     * default: `() => {}`
     */
    disabledDay?: ((date: Date) => boolean) | any;
    /** default: `date` */
    displayMode?: 'dateRange' | 'date' | any;
    /** default: `true` */
    dragSelectionEnabled?: boolean | any;
    /** default: `false` */
    editableDateInputs?: boolean | any;
    /** default: `Continuous` */
    endDatePlaceholder?: string | any;
    /** default: `false` */
    fixedHeight?: boolean | any;
    /**
     * Which range and step are focused. First value is index of ranges, second value
     * is which step on date range (startDate or endDate)
     *
     * default: `[0, 0]`
     */
    focusedRange?: RangeFocus | any;
    /**
     * Initial value for focused range. See `focusedRange` for usage
     *
     * default: none
     */
    initialFocusedRange?: RangeFocus | any;
    /**
     * default: `en-US` from `date-fns/locale`
     *
     * Complete list here: https://github.com/hypeserver/react-date-range/blob/next/src/locale/index.js
     */
    locale?: Locale | any;
    /** default: 20 years after the current date */
    maxDate?: Date | any;
    /** default: 100 years before the current date */
    minDate?: Date | any;
    /** default: `MMM yyyy` */
    monthDisplayFormat?: string | any;
    /** default: `1` */
    months?: number | any;
    /**
     * Custom renderer function for the month and year navigation section
     *
     * default: none
     */
    navigatorRenderer?:
        | ((
              currFocusedDate: Date,
              changeShownDate: (
                  value: Date | number | string,
                  mode?: 'set' | 'setYear' | 'setMonth' | 'monthOffset',
              ) => void,
              props: CalendarProps,
          ) => JSX.Element)
        | any;
    /** default: none */
    onChange?: ((date: Date) => void) | any;
    /** default: none */
    onPreviewChange?: ((previewDate?: Date) => void) | any;
    /** default: none */
    onRangeFocusChange?: ((newFocusedRange: RangeFocus) => void) | any;
    /** default: none */
    onShownDateChange?: ((date: Date) => void) | any;
    /** default: false */
    preventSnapRefocus?: boolean | any;
    /** default: none */
    preview?: Preview | any;
    /** default: `['#3d91ff', '#3ecf8e', '#fed14c']` */
    rangeColors?: string[] | any;
    /** default: [] */
    ranges?: Range[];
    /**
     * Custom scroll options for various parts of the display
     *
     * default: `{ enabled: false }`
     */
    scroll?: ScrollOptions | any;
    /** default: true */
    showDateDisplay?: boolean | any;
    /** default: true */
    showMonthAndYearPickers?: boolean | any;
    /** default: true */
    showMonthArrow?: boolean | any;
    /** default: true */
    showPreview?: boolean | any;
    /** default: none */
    shownDate?: Date | any;
    /** default: `Early` */
    startDatePlaceholder?: string | any;
    /** default: none */
    updateRange?: ((newRange: Range) => void) | any;
    /** default: `E` */
    weekdayDisplayFormat?: string | any;
    /** default: none */
    weekStartsOn?: 0 | 1 | 2 | 3 | 4 | 5 | 6 | any;
}

export class Calendar extends React.Component<CalendarProps> {}

// =============================================================================
// DateRange Component
// =============================================================================

export interface DateRangeProps extends Omit<CalendarProps, 'onChange'> {
    /** default: none */
    onChange?: ((rangesByKey: RangeKeyDict) => void) | any;
    /** default: `false` */
    moveRangeOnFirstSelection?: boolean;
    /** default: `false` */
    retainEndDateOnFirstSelection?: boolean;
}

export class DateRange extends React.Component<DateRangeProps> {}

// =============================================================================
// DefinedRange Component and Defaults
// =============================================================================

export interface StaticRange {
    range: (props?: DefinedRangeProps) => Preview;
    isSelected: (range: Range) => boolean;
    label?: string | any;
    hasCustomRendering?: boolean | any;
}

export interface InputRange {
    range: (value: number, props?: DefinedRangeProps) => Range;
    getCurrentValue: (range: Range) => string | number;
    label?: string | any;
}

export interface DefinedRangeProps {
    /** default: none */
    className?: string | any;
    /**
     * Which range and step are focused. First value is index of ranges, second value
     * is which step on date range (startDate or endDate)
     *
     * default: `[0, 0]`
     */
    focusedRange?: RangeFocus | any;
    /** default: none */
    footerContent?: React.ReactNode | any;
    /** default: none */
    headerContent?: React.ReactNode | any;
    /** default: `defaultInputRanges` (defined and exported in package) */
    inputRanges?: InputRange[] | any;
    /** default: none */
    onChange?: ((rangesByKey: RangeKeyDict) => void) | any;
    /** default: none */
    onPreviewChange?: ((preview?: Preview) => void) | any;
    /** default: `['#3d91ff', '#3ecf8e', '#fed14c']` */
    rangeColors?: string[] | any;
    /** default: [] */
    ranges?: Range[] | any;
    /**
     * Custom renderer function for static range labels
     *
     * default: none
     */
    renderStaticRangeLabel?: ((staticRange: StaticRange) => React.ReactNode) | any;
    /** default: `defaultStaticRanges` (defined and exported in package) */
    staticRanges?: StaticRange[] | any;
}

export class DefinedRange extends React.Component<DefinedRangeProps> {}

export const defaultStaticRanges: StaticRange[];
export const defaultInputRanges: InputRange[];

export function createStaticRanges(ranges: StaticRange[]): StaticRange[];

// =============================================================================
// DateRangePicker Component
// =============================================================================

export interface DateRangePickerProps extends DateRangeProps, DefinedRangeProps {
    /**
     * NOTE: currently the date range picker component passes the `onPreviewChange` prop
     * (if included) to both subcomponents even though they seemingly expect different types
     * for this method. Therefore, if passing this prop, it should be able to handle both
     * types so it will work for either component.
     */
    /** default: none */
    onPreviewChange?: ((preview?: Date | Preview) => void) | any;
}

export class DateRangePicker extends React.Component<DateRangePickerProps> {}
